﻿/* 
 * Copyright (C) 2008 Alan Ross, Tojio GmbH (http://lab.tojio.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
 

package
{	
	import flash.display.*;
	import flash.events.*;
	import flash.text.TextField;
	//import Papervision (Great White)
	import org.papervision3d.cameras.Camera3D;
	import org.papervision3d.materials.BitmapMaterial;
	import org.papervision3d.materials.WireframeMaterial;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.view.BasicView;
	//import WiiFlash (0.4)
	import org.wiiflash.*;
	import org.wiiflash.events.*;

    //[SWF(frameRate="31", width="900", height="600", backgroundColor="0x111111")]
    public class WiiFlash3DObject extends Sprite
    {
    	//papervision
        private var view:BasicView;
        private var rootNode:DisplayObject3D;
        private var environment:Cube;
        
        //Play around with this value to get rid of the 'noise' the Wiimote sends
        //I guess each Wiimote will have its own optimal value here... 
		private const THRESHOLD:Number = .08;
		
		private var newRotX:Number=0;
		private var oldRotX:Number=0;
		private var newRotY:Number=0;
		private var oldRotY:Number=0;
		private var newRotZ:Number=0;
		private var oldRotZ:Number=0;
		
		// create a new Wiimote
		private var wiimote:Wiimote;
        
		//debugging
		private var debug:TextField;
		
		//________________________________________________________
		// constructor
		//________________________________________________________
		public function WiiFlash3DObject()
		{   
            super();
			if(stage != null){ init(); }
			else{ addEventListener(Event.ADDED_TO_STAGE, init); }
		}
		
		
		//________________________________________________________
		// initialize
		//________________________________________________________
		public function init(e:Event=null):void 
		{	
			stage.scaleMode = StageScaleMode.NO_SCALE;
        	stage.align = StageAlign.TOP_LEFT;
        	stage.quality = StageQuality.HIGH;
        	
        	debug = new TextField(); debug.height = 400; debug.width = 300; 
        	debug.x = debug.y = 10; debug.textColor = 0xffffff; debug.selectable = false; 
        	addChild(debug);
        	
        	initWiiFlash();
        	init3D();
        	
        	addEventListener( Event.ENTER_FRAME, loop );
		}
		
		
		//________________________________________________________
		// setup papervision
		//________________________________________________________
		private function init3D():void
		{
			view = new BasicView(stage.stageWidth, stage.stageHeight, true, true);
        	view.camera.zoom = 12;
        	view.camera.focus = 100;
        	addChildAt(view,0);
	
			//the root node
			rootNode = new DisplayObject3D();
			
			//the environment
			var m:WireframeMaterial = new WireframeMaterial(0x333333,.2);
			var ml:MaterialsList = new MaterialsList({front:m, back:m, right:m, left:m, top:m, bottom:m});
			var s:uint = 700;	//size
			var q:uint = 10;	//quality
			environment = new Cube(ml,s,s,s,q,q,q,Cube.ALL);
			view.scene.addChild(environment, "environment");
			
			//now add a demo object the Wiimote can play with...
			var bmd:BitmapData = new BitmapData(1000,1000);
			bmd.perlinNoise(10,10,3,Math.random()*10,true,true);
			var bm:BitmapMaterial = new BitmapMaterial(bmd);
			
			rootNode.addChild(new Cube(new MaterialsList({all:bm}),100,10,100,1,1,1));
			
        	view.scene.addChild(rootNode);
		}
		
		//________________________________________________________
		// setup papervision
		//________________________________________________________
		private function initWiiFlash():void
		{
			wiimote = new Wiimote();
			wiimote.connect();
			
			wiimote.addEventListener( Event.CONNECT, onWiimoteConnect );
			wiimote.addEventListener( IOErrorEvent.IO_ERROR, onWiimoteConnectError );
			wiimote.addEventListener( Event.CLOSE, onCloseConnection );
			wiimote.addEventListener( WiimoteEvent.UPDATE, onUpdated );
		}
		
		//________________________________________________________
		// main rendering loop
		//________________________________________________________
        private function loop( event : Event ):void
        {
        	//Use mouse to fly around a little
        	Camera3D(view.camera).hover(0, view.viewport.containerSprite.mouseX*-.004, view.viewport.containerSprite.mouseY*.004);
        	//update rotation and pos of 3d object
			update3DObject();
			//render all changes done
			view.singleRender();
        }

	
		//________________________________________________________
		// update rotation and pos of 3d object
		//________________________________________________________
       	private function update3DObject():void
		{
			newRotX = wiimote.pitch;
        	newRotY = wiimote.yaw;
        	newRotZ = wiimote.roll;
        	
        	//compare difference of last value to this one
        	// if it is bigger than the threshold, use it
        	if(Math.abs(newRotX-oldRotX) > THRESHOLD)
        	{
        		newRotX = (newRotX+oldRotX)/2;
        		rootNode.rotationX = 180/Math.PI*-newRotX;
        	}
        	if(Math.abs(newRotY-oldRotY) > THRESHOLD)
        	{
        		newRotY = (newRotY+oldRotY)/2;
        		//rootNode.rotationY = 180/Math.PI*-newRotY;
        	}
        	if(Math.abs(newRotZ-oldRotZ) > THRESHOLD)
        	{
        		newRotZ = (newRotZ+oldRotZ)/2;
        		rootNode.rotationZ = 180/Math.PI*-newRotZ;
        	}
        	
        	oldRotX = newRotX;
        	oldRotY = newRotY;
        	oldRotZ = newRotZ;
        
			if(wiimote.plus) rootNode.scale += .1;
			if(wiimote.minus) rootNode.scale -= .1;
			if(wiimote.left) rootNode.x += 10;
			if(wiimote.right) rootNode.x -= 10;
			if(wiimote.down) rootNode.y += 10;
			if(wiimote.up) rootNode.y -= 10;
		}

		// just some info from here on...
		private function onCloseConnection( e:Event ):void
		{ debug.text = "Connection closed, WiiFlash Server has been closed"; }
		
		private function onWiimoteConnectError( e:IOErrorEvent ):void
		{ debug.text = "Couldn't connect, make sure WiiFlash Server is running !"; }
	
		private function onWiimoteConnect( e:Event ):void
		{ debug.text= "Wiimote successfully connected"; }
	
		private function onUpdated( e:WiimoteEvent ):void 
		{
			debug.text = "Wiimote: "+wiimote.id+"\n";
			debug.appendText("Sensor X : " + wiimote.sensorX + "\n");
			debug.appendText("Sensor Y : " + wiimote.sensorY + "\n");
			debug.appendText("Sensor Z : " + wiimote.sensorZ + "\n");
			debug.appendText("Pitch : " + wiimote.pitch + "\n");
			debug.appendText("Roll : " + wiimote.roll + "\n");
			debug.appendText("Yaw : " + wiimote.yaw + "\n");
			
			debug.appendText("Button UP: " + wiimote.up + "\n");
			debug.appendText("Button DOWN: " + wiimote.down + "\n");
			debug.appendText("Button RIGHT: " + wiimote.right + "\n");
			debug.appendText("Button LEFT: " + wiimote.left + "\n");
			debug.appendText("Button PLUS: " + wiimote.plus + "\n");
			debug.appendText("Button MINUS: " + wiimote.minus + "\n");
		}
    }
}